K_TRIGGER_PROCESS_ON_CREATE = 'PROCESS_ON_CREATE'
K_TRIGGER_PROCESS_ON_UPDATE = 'PROCESS_ON_UPDATE'
K_TRIGGER_ACTIVITY_ON_CREATE = 'ACTIVITY_ON_CREATE'
K_TRIGGER_ACTIVITY_ON_UPDATE = 'ACTIVITY_ON_UPDATE'
K_TRIGGER_ACTIVITY_ON_COMPLETE = 'ACTIVITY_ON_COMPLETE'
K_TRIGGER_ACTIVITY_ON_DELETE = 'ACTIVITY_ON_DELETE'
K_TRIGGER_ACTIVITY_SUBMISSION_ON_SUBMIT = 'ACTIVITY_SUBMISSION_ON_SUBMIT'
K_TRIGGER_ACTIVITY_SUBMISSION_ON_UNSUBMIT = 'ACTIVITY_SUBMISSION_ON_UNSUBMIT'
K_TRIGGER_ACTIVITIES_REMINDER_ON_SEND = 'ACTIVITIES_REMINDER_ON_SEND'


K_SUBMISSION_TYPE_NOTE = 'Note'
K_SUBMISSION_TYPE_NOTE_AND_DATE = 'NoteAndDate'
K_SUBMISSION_TYPE_NOTE_AND_MULTIPLE_FILE = 'NoteAndMultipleFile'
K_SUBMISSION_TYPE_NOTE_AND_YES_NO = 'NoteAndYesNo'

K_PROCESS_STATUS_COMPLETED = 'Completed'
K_PROCESS_STATUS_CANCELLED = 'Cancelled'
K_PROCESS_STATUS_ON_HOLD = 'On Hold'
K_PROCESS_STATUS_ON_PROGRESS = 'On Progress'
K_PROCESS_STATUS_IGNORED = 'Ignored'
